/* Standard headers. */
#include <filesystem>
#include <thread>
#include <unistd.h>

/* Module headers. */
#include <utils/include/edgeai_perfstats.h>

/*
 * Outputs log files containing real-time performance metrics.
 * Log files generated with performance metrics averaged over some fixed time interval.
 */

// bool ti::edgeai::utils::m_printstats = false;
namespace ti::utils
{
    bool m_printstats;
    app_perf_point_t perf;

    static void Loop()
    {
        app_perf_point_t *perf_arr[1];

        /* Create empty log directory (created as edge_ai_apps/perf_logs) */
        char tem[40] = "../perf_logs";              // assumes that binary was run from apps_cpp
        std::filesystem::remove_all(tem);
        std::filesystem::create_directories(tem);

        /* Set prefix for log files as "Log". cur always points to the character after "Log" */
        char buf[20] = "";
        char *cur = buf, *const end = buf + sizeof buf;            
        cur += snprintf(cur, end-cur, "%s", "Log");

        /* Set prefix for the log file to be deleted */
        char delete_file[40] = "";
        char *del_file_cur = delete_file, *const del_file_end = delete_file + sizeof delete_file;            
        del_file_cur += snprintf(del_file_cur, del_file_end-del_file_cur, "%s", "../perf_logs/Log");
            
        int logNumber=0;
        int save_history = 16;      // Defines how many log files to keep at a time
        while (m_printstats)
        {
            perf_arr[0] = &perf;

            /* If (number of log files > save_history) : delete oldest file */
            if (logNumber>=save_history)
            {
                snprintf(del_file_cur, del_file_end-del_file_cur, "%d%s", logNumber-save_history, ".md");
                bool del_status = std::filesystem::remove(delete_file);
            }
            
            /* Incrementally name the log file : "Log<logNumber>".md */
            snprintf(cur, end-cur, "%d", logNumber++);

            FILE *fp = appPerfStatsExportOpenFile(tem,buf);
            if((NULL != fp))
            {
                ::appPerfStatsExportAll(fp, perf_arr, 1);
                ::appPerfStatsExportCloseFile(fp);
                ::appPerfStatsResetAll();
            }
            else
            {
                printf("fp is null\n");
            }
            // refresh();

            /* Log files generated every 2s */
            this_thread::sleep_for(chrono::milliseconds(2000));
        }
    }

    // Starts thread to log performance files
    void enableReport(bool state)
    {
        m_printstats = state;
        if (state)
        {
            thread threadId = thread(Loop);
            threadId.detach();
        }
    }

    // When called, start recording and averaging performance metrics
    void startRec()
    {
        appPerfPointBegin(&perf);
    }

    // When called, pause recording and averaging performance metrics
    void endRec()
    {
        appPerfPointEnd(&perf);
    }

    // Stop performance logs
    void disableReport()
    {
        m_printstats = false;
        sleep(1);
    }
}
