#!/bin/bash

#  Copyright (C) 2021 Texas Instruments Incorporated - http://www.ti.com/
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#
#    Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#    Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
#
#    Neither the name of Texas Instruments Incorporated nor the names of
#    its contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Putting display on standby
killall weston 2>/dev/null

# Logic here tries to ping an internal server and sets the proxy if it succeeds
# Modify the server and proxy URLs as requied
ping bitbucket.itg.ti.com -c 1 > /dev/null 2>&1
if [ "$?" -eq "0" ]; then
    export HTTPS_PROXY=http://webproxy.ext.ti.com:80
    export https_proxy=http://webproxy.ext.ti.com:80
    export HTTP_PROXY=http://webproxy.ext.ti.com:80
    export http_proxy=http://webproxy.ext.ti.com:80
    export ftp_proxy=http://webproxy.ext.ti.com:80
    export FTP_PROXY=http://webproxy.ext.ti.com:80
    export no_proxy=ti.com,localhost
    # configure local ntpd servers
    cat << EOF > /etc/ntpd.conf
server bdtime.itg.ti.com
server fltime.itg.ti.com
server sbtime.itg.ti.com
server letime.itg.ti.com
EOF
else
    cat << EOF > /etc/ntpd.conf
servers pool.ntp.org
EOF
fi

#Set time
ntpd -s

# Disable Neo-DLR phone-home feature
echo '{"enable_phone_home": false}' > /usr/lib/python3.8/site-packages/dlr/counter/ccm_config.json

# Link headers and libraries for DLR
mkdir -p /usr/dlr/
ln -snf /usr/lib/python3.8/site-packages/dlr/libdlr.so /usr/dlr/libdlr.so
ln -snf /usr/dlr/libdlr.so /usr/lib/libdlr.so

# Link libraries and python packages for ONNXRT
ln -snf /usr/lib/libonnxruntime.so.1.7.0 /usr/lib/libonnxruntime.so

rm -rf /usr/lib/libvx_tidl_rt.so.map
rm -rf /usr/lib/libtidl_onnxrt_EP.so.map
rm -rf /usr/lib/libtidl_tfl_delegate.so.map

ldconfig

#Check for usb and csi cameras present and modify config files accordingly
ls /dev/v4l/by-path/*csi*video-index0 > /dev/null 2>&1
if [ "$?" == "0" ]; then
    CSI_CAM=`ls /dev/v4l/by-path/*csi*video-index0 | head -1 | xargs readlink -f`
    for i in `ls /sys/bus/i2c/devices/`
    do
        cat /sys/bus/i2c/devices/$i/name | grep ov5640 > /dev/null
        if [ "$?" -eq "0" ]; then
            I2C_ENUM=$i
            break
        fi
    done
    media-ctl --set-v4l2 '"ov5640 '$i'":0 [fmt:UYVY8_2X8/1280x720@1/30]'
fi

ls /dev/v4l/by-path/*usb*video-index0 > /dev/null 2>&1
if [ "$?" == "0" ]; then
    USB_CAM=`ls /dev/v4l/by-path/*usb*video-index0 | head -1 | xargs readlink -f`
    USB_CAM_ESC=${USB_CAM//\//\\\/}
    sed -i "s/\/dev\/video[0-9]/$USB_CAM_ESC/g" configs/*
fi

# Enable remote core logging
ls /opt/vx_app_arm_remote_log.out > /dev/null 2>&1
if [ "$?" == "0" ]; then
    /opt/vx_app_arm_remote_log.out > /opt/remote_core_handshake_log.txt 2>&1 &
fi
