#!/bin/bash

#  Copyright (C) 2021 Texas Instruments Incorporated - http://www.ti.com/
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
#
#    Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
#    Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the
#    distribution.
#
#    Neither the name of Texas Instruments Incorporated nor the names of
#    its contributors may be used to endorse or promote products derived
#    from this software without specific prior written permission.
#
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
#  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
#  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
#  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
#  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
#  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
#  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
#  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

topdir=`git rev-parse --show-toplevel`
usb_camera=`ls /dev/v4l/by-path/*usb*video-index0 | head -1 | xargs readlink -f`
usb_fmt=jpeg
csi_camera=`ls /dev/v4l/by-path/*csi*video-index0 | head -1 | xargs readlink -f`
csi_fmt=auto

BRIGHTWHITE='\033[0;37;1m'
RED='\033[0;31m'
GREEN='\033[0;32m'
NOCOLOR='\033[0m'

####################################################################################################

export TEST_ENGINE_DEBUG=1
config_file="$topdir/tests/test_config.yaml"
parse_script="$topdir/tests/parse_data_sheet.py"
timeout=30
filter=" grep -f /tmp/test_models.txt"

# This will make sure that the test is run only on the listed models

cat > /tmp/test_models.txt << EOF
TFL-CL-0000-mobileNetV1-mlperf
TVM-CL-3410-gluoncv-mxnet-mobv2
ONR-CL-6150-mobileNetV2-1p4-qat
TFL-OD-2020-ssdLite-mobDet-DSP-coco-320x320
EOF

sed -i '/^$/d' /tmp/test_models.txt

####################################################################################################
cleanup() {
	echo
	echo "[Ctrl-C] Killing the script..."
	ps -eaf | grep "[g]en_data_sheet.sh" | awk -F" " '{print $2}' | xargs kill -9
}

# TODO: The trap is not reliable currently, need to be fixed
trap cleanup SIGINT

rm -rf ../perf_logs/

for suite in "CPP-PERF"; do
	for input in "USBCAM" "VIDEO"; do
	for output in "DISPLAY"; do
		test_suite="$suite-$input-$output"

		# Update the config file with correct parameters
		# TODO: Simplify the complex find and replace with yq
		case $input in
		"VIDEO")
			sed -i "s@source:.*@source: $topdir/data/videos/video_0000.mp4@" $config_file
			;;
		"USBCAM")
			sed -i "s@source:.*@source: $usb_camera@" $config_file
			sed -i "s@format:.*@format: $usb_fmt@" $config_file
			;;
		"CSICAM")
			sed -i "s@source:.*@source: $csi_camera@" $config_file
			sed -i "s@format:.*@format: $csi_fmt@" $config_file
			;;
		*)
			continue
			;;
		esac

		case $output in
		"VIDEO")
			sed -i "s@sink:.*@sink: $topdir/data/videos/output_0000.avi@" $config_file
			;;
		"DISPLAY")
			sed -i "s@sink:.*@sink: kmssink@" $config_file
			;;
		*)
			continue
			;;
		esac

        cat > data_sheet_$test_suite.rst << EOF
.. csv-table:: Performance table
    :header: "Model", "FPS", "Total time (ms)", "Inference time (ms)", "A72 Load (%)", "DDR Read BW (MB/s)", "DDR Write BW (MB/s)", "DDR Total BW (MB/s)", "C71 Load (%)", "C66_1 Load (%)", "C66_2 Load (%) ", "MCU2_0 Load (%)", "MCU2_1 Load (%)", "MSC_0 (%)", "MSC_1 (%)", "VISS (%)", "NF (%)", "LDC (%)", "SDE (%)", "DOF (%)"
    :widths: 50, 20, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10, 10

EOF
		# Now start the test engine with above filter
		./test_engine.sh $test_suite $config_file $timeout $parse_script "$filter"
	done
	done
done

#kill using following command
#ps -eaf | grep "[s]anity_test.sh" | awk -F" " '{print $2}' | xargs kill -9
